@@ -8,36 +8,18 @@
 
 #include "main.h"
 
-#include "ADTSReader.h"
-#include "Session.h"
+#include "CompKodiProps.h"
+#include "SrvBroker.h"
 #include "Stream.h"
-#include "TSReader.h"
-#include "WebmReader.h"
-#include "parser/DASHTree.h"
-#include "parser/HLSTree.h"
-#include "parser/SmoothTree.h"
-#include "samplereader/ADTSSampleReader.h"
-#include "samplereader/FragmentedSampleReader.h"
-#include "samplereader/SubtitleSampleReader.h"
-#include "samplereader/TSSampleReader.h"
-#include "samplereader/WebmSampleReader.h"
-#include "utils/Utils.h"
+#include "samplereader/SampleReaderFactory.h"
 #include "utils/log.h"
-#include "utils/PropertiesUtils.h"
 
-#include <stdarg.h> // va_list, va_start, va_arg, va_end
-
-using namespace UTILS;
+using namespace PLAYLIST;
 using namespace SESSION;
 
-static const AP4_Track::Type TIDC[adaptive::AdaptiveTree::STREAM_TYPE_COUNT] = {
-    AP4_Track::TYPE_UNKNOWN, AP4_Track::TYPE_VIDEO, AP4_Track::TYPE_AUDIO,
-    AP4_Track::TYPE_SUBTITLES };
-
 CInputStreamAdaptive::CInputStreamAdaptive(const kodi::addon::IInstanceInfo& instance)
   : CInstanceInputStream(instance)
 {
-  memset(m_IncludedStreams, 0, sizeof(m_IncludedStreams));
 }
 
 ADDON_STATUS CInputStreamAdaptive::CreateInstance(const kodi::addon::IInstanceInfo& instance,
@@ -56,48 +38,13 @@ bool CInputStreamAdaptive::Open(const kodi::addon::InputstreamProperty& props)
   LOG::Log(LOGDEBUG, "Open()");
 
   std::string url = props.GetURL();
-  m_kodiProps = PROPERTIES::ParseKodiProperties(props.GetProperties());
 
-  if (m_kodiProps.m_manifestType == PROPERTIES::ManifestType::UNKNOWN)
-    return false;
+  CSrvBroker::GetInstance()->Init(props.GetProperties());
 
-  std::uint8_t drmConfig{0};
-  if (m_kodiProps.m_isLicensePersistentStorage)
-    drmConfig |= SSD::SSD_DECRYPTER::CONFIG_PERSISTENTSTORAGE;
-
-  // If the manifest URL contains headers then replace the manifest headers set with property
-  //! @todo: remove pipe support on Kodi v21
-  size_t posHeader = url.find("|");
-  if (posHeader != std::string::npos)
-  {
-    LOG::Log(LOGWARNING, "Set headers to the manifest url by using pipe \"|\" char is deprecated, "
-                         "and will be removed in future.\n"
-                         "Use \"inputstream.adaptive.manifest_headers\" and "
-                         "\"inputstream.adaptive.stream_headers\" properties instead.");
-    m_kodiProps.m_manifestHeaders.clear();
-    ParseHeaderString(m_kodiProps.m_manifestHeaders, url.substr(posHeader + 1));
-    url = url.substr(0, posHeader);
-
-    if (m_kodiProps.m_streamHeaders.empty())
-      m_kodiProps.m_streamHeaders = m_kodiProps.m_manifestHeaders;
-  }
-
-  //! @todo: remove this old forced behaviour on Kodi v21
-  if (m_kodiProps.m_manifestHeaders.empty() && !m_kodiProps.m_streamHeaders.empty())
-  {
-    LOG::Log(
-        LOGWARNING,
-        "Set headers to the manifest by using \"inputstream.adaptive.stream_headers\" property "
-        "is a deprecated behaviour that will be removed in future.\n"
-        "To set headers to the manifest, use \"inputstream.adaptive.manifest_headers\" property.");
-    m_kodiProps.m_manifestHeaders = m_kodiProps.m_streamHeaders;
-  }
-
-  m_session = std::make_shared<CSession>(m_kodiProps, url, props.GetProfileFolder());
+  m_session = std::make_shared<CSession>(url);
   m_session->SetVideoResolution(m_currentVideoWidth, m_currentVideoHeight, m_currentVideoMaxWidth,
                                 m_currentVideoMaxHeight);
 
-  m_session->SetDrmConfig(drmConfig);
   if (!m_session->Initialize())
   {
     m_session = nullptr;
@@ -119,7 +66,7 @@ bool CInputStreamAdaptive::GetStreamIds(std::vector<unsigned int>& ids)
 
   if (m_session)
   {
-    adaptive::AdaptiveTree::Period* period;
+    CPeriod* period;
     int period_id = m_session->GetPeriodId();
     iids.m_streamCount = 0;
     unsigned int id;
@@ -134,26 +81,27 @@ bool CInputStreamAdaptive::GetStreamIds(std::vector<unsigned int>& ids)
         continue;
       }
 
-      uint8_t cdmId(static_cast<uint8_t>(stream->m_adStream.getRepresentation()->pssh_set_));
-      if (stream->m_isValid &&
-          (m_session->GetMediaTypeMask() & static_cast<uint8_t>(1) << stream->m_adStream.get_type()))
+      uint8_t cdmId(static_cast<uint8_t>(stream->m_adStream.getRepresentation()->m_psshSetPos));
+      if (stream->m_isValid && (m_session->GetMediaTypeMask() &
+                                static_cast<uint8_t>(1) << static_cast<int>(stream->m_adStream.GetStreamType())))
       {
         if (m_session->GetMediaTypeMask() != 0xFF)
         {
-          const adaptive::AdaptiveTree::Representation* rep(stream->m_adStream.getRepresentation());
-          if (rep->flags_ & adaptive::AdaptiveTree::Representation::INCLUDEDSTREAM)
+          const CRepresentation* rep = stream->m_adStream.getRepresentation();
+          if (rep->IsIncludedStream())
             continue;
         }
         if (m_session->IsLive())
         {
           period = stream->m_adStream.getPeriod();
-          if (period->sequence_ == m_session->GetInitialSequence())
+          if (m_session->HasInitialSequence() &&
+              period->GetSequence() == m_session->GetInitialSequence())
           {
             id = i + 1000;
           }
           else
           {
-            id = i + (period->sequence_ + 1) * 1000;
+            id = i + (period->GetSequence() + 1) * 1000;
           }
         }
         else
@@ -188,30 +136,6 @@ bool CInputStreamAdaptive::GetStream(int streamid, kodi::addon::InputstreamInfo&
 
   if (stream)
   {
-    uint8_t cdmId(static_cast<uint8_t>(stream->m_adStream.getRepresentation()->pssh_set_));
-    if (stream->m_isEncrypted && m_session->GetCDMSession(cdmId) != nullptr)
-    {
-      kodi::addon::StreamCryptoSession cryptoSession;
-
-      LOG::Log(LOGDEBUG, "GetStream(%d): initalizing crypto session", streamid);
-      cryptoSession.SetKeySystem(m_session->GetCryptoKeySystem());
-
-      const char* sessionId(m_session->GetCDMSession(cdmId));
-      cryptoSession.SetSessionId(sessionId);
-
-      if (m_session->GetDecrypterCaps(cdmId).flags &
-          SSD::SSD_DECRYPTER::SSD_CAPS::SSD_SUPPORTS_DECODING)
-        stream->m_info.SetFeatures(INPUTSTREAM_FEATURE_DECODE);
-      else
-        stream->m_info.SetFeatures(0);
-
-      cryptoSession.SetFlags((m_session->GetDecrypterCaps(cdmId).flags &
-                          SSD::SSD_DECRYPTER::SSD_CAPS::SSD_SECURE_DECODER)
-                             ? STREAM_CRYPTO_FLAG_SECURE_DECODER
-                             : 0);
-      stream->m_info.SetCryptoSession(cryptoSession);
-    }
-
     info = stream->m_info;
     return true;
   }
@@ -227,9 +151,11 @@ void CInputStreamAdaptive::UnlinkIncludedStreams(CStream* stream)
     if (mainStream->GetReader())
       mainStream->GetReader()->RemoveStreamType(stream->m_info.GetStreamType());
   }
-  const adaptive::AdaptiveTree::Representation* rep(stream->m_adStream.getRepresentation());
-  if (rep->flags_ & adaptive::AdaptiveTree::Representation::INCLUDEDSTREAM)
-    m_IncludedStreams[stream->m_info.GetStreamType()] = 0;
+
+  const CRepresentation* rep = stream->m_adStream.getRepresentation();
+
+  if (rep->IsIncludedStream())
+    m_IncludedStreams.erase(stream->m_info.GetStreamType());
 }
 
 void CInputStreamAdaptive::EnableStream(int streamid, bool enable)
@@ -253,6 +179,14 @@ void CInputStreamAdaptive::EnableStream(int streamid, bool enable)
 bool CInputStreamAdaptive::OpenStream(int streamid)
 {
   LOG::Log(LOGDEBUG, "OpenStream(%d)", streamid);
+  // This method can be called when:
+  // - Stream first start
+  // - Chapter/period change
+  // - Automatic stream (representation) quality change (such as adaptive)
+  // - Manual stream (representation) quality change (from OSD)
+  // streamid behaviour:
+  // - The streamid can be influenced by Kodi core based on preferences set in Kodi settings (e.g. language)
+  // - Fallback calls, e.g. if the opened video streamid fails, Kodi core will try to open another video streamid
 
   if (!m_session)
     return false;
@@ -274,14 +208,19 @@ bool CInputStreamAdaptive::OpenStream(int streamid)
       return false;
   }
 
-  bool needRefetch = false; //Make sure that Kodi fetches changes
   stream->m_isEnabled = true;
 
-  adaptive::AdaptiveTree::Representation* rep = stream->m_adStream.getRepresentation();
+  //! @todo: for live with multiple periods (like HLS DISCONTINUITIES) for subtitle case
+  //! when the subtitle has been disabled from video start, and happens a period change,
+  //! kodi call OpenStream to the subtitle stream as if it were enabled, and disable it with EnableStream
+  //! just after it, this lead to log error "GenerateSidxSegments: [AS-x] Cannot generate segments from SIDX on repr..."
+  //! need to find a solution to avoid open the stream when previously disabled from previous period
+
+  CRepresentation* rep = stream->m_adStream.getRepresentation();
 
   // If we select a dummy (=inside video) stream, open the video part
   // Dummy streams will be never enabled, they will only enable / activate audio track.
-  if (rep->flags_ & adaptive::AdaptiveTree::Representation::INCLUDEDSTREAM)
+  if (rep->IsIncludedStream())
   {
     CStream* mainStream;
     stream->m_mainId = 0;
@@ -309,131 +248,89 @@ bool CInputStreamAdaptive::OpenStream(int streamid)
     return false;
   }
 
-  if (rep->flags_ & adaptive::AdaptiveTree::Representation::SUBTITLESTREAM)
-  {
-    stream->SetReader(std::make_unique<CSubtitleSampleReader>(
-        rep->url_, streamid, stream->m_info.GetCodecInternalName(),
-        stream->m_adStream.GetStreamParams(), stream->m_adStream.GetStreamHeaders()));
-    return stream->GetReader()->GetInformation(stream->m_info);
-  }
+  m_session->PrepareStream(stream);
 
-  AP4_Movie* movie(m_session->PrepareStream(stream, needRefetch));
-
-  // We load fragments on PrepareTime for HLS manifests and have to reevaluate the start-segment
-  //if (m_session->GetManifestType() == PROPERTIES::ManifestType::HLS)
-  //  stream->m_adStream.restart_stream();
   stream->m_adStream.start_stream(m_lastPts);
+  stream->SetAdByteStream(std::make_unique<CAdaptiveByteStream>(&stream->m_adStream));
 
-  if (rep->containerType_ == adaptive::AdaptiveTree::CONTAINERTYPE_TEXT)
-  {
-    stream->SetAdByteStream(std::make_unique<CAdaptiveByteStream>(&stream->m_adStream));
-    stream->SetReader(std::make_unique<CSubtitleSampleReader>(stream, streamid,
-                                                             stream->m_info.GetCodecInternalName()));
-  }
-  else if (rep->containerType_ == adaptive::AdaptiveTree::CONTAINERTYPE_TS)
-  {
-    stream->SetAdByteStream(std::make_unique<CAdaptiveByteStream>(&stream->m_adStream));
+  ContainerType reprContainerType = rep->GetContainerType();
+  uint32_t mask = (1U << stream->m_info.GetStreamType()) | m_session->GetIncludedStreamMask();
+  auto reader = ADP::CreateStreamReader(reprContainerType, stream, static_cast<uint32_t>(streamid), mask);
 
-    uint32_t mask{(1U << stream->m_info.GetStreamType()) | m_session->GetIncludedStreamMask()};
-    stream->SetReader(std::make_unique<CTSSampleReader>(
-        stream->GetAdByteStream(), stream->m_info.GetStreamType(), streamid, mask));
-
-    if (stream->GetReader()->Initialize())
-    {
-      m_session->OnSegmentChanged(&stream->m_adStream);
-    }
-    else if (stream->m_adStream.get_type() == adaptive::AdaptiveTree::AUDIO)
-    {
-      // If TSSampleReader fail, try fallback to ADTS
-      //! @todo: we should have an appropriate file type check
-      //! e.g. with HLS we determine the container type from file extension
-      //! in the url address, but .ts file could have ADTS
-      LOG::LogF(LOGWARNING, "Cannot initialize TS sample reader, fallback to ADTS sample reader");
-      rep->containerType_ = adaptive::AdaptiveTree::CONTAINERTYPE_ADTS;
-
-      stream->GetAdByteStream()->Seek(0); // Seek because bytes are consumed from previous reader
-      stream->SetReader(std::make_unique<CADTSSampleReader>(stream->GetAdByteStream(), streamid));
-    }
-    else
-    {
-      stream->Disable();
-      return false;
-    }
-  }
-  else if (rep->containerType_ == adaptive::AdaptiveTree::CONTAINERTYPE_ADTS)
-  {
-    stream->SetAdByteStream(std::make_unique<CAdaptiveByteStream>(&stream->m_adStream));
-    stream->SetReader(std::make_unique<CADTSSampleReader>(stream->GetAdByteStream(), streamid));
-  }
-  else if (rep->containerType_ == adaptive::AdaptiveTree::CONTAINERTYPE_WEBM)
-  {
-    stream->SetAdByteStream(std::make_unique<CAdaptiveByteStream>(&stream->m_adStream));
-    stream->SetReader(std::make_unique<CWebmSampleReader>(stream->GetAdByteStream(), streamid));
-    if (!stream->GetReader()->Initialize())
-    {
-      stream->Disable();
-      return false;
-    }
-  }
-  else if (rep->containerType_ == adaptive::AdaptiveTree::CONTAINERTYPE_MP4)
-  {
-    stream->SetAdByteStream(std::make_unique<CAdaptiveByteStream>(&stream->m_adStream));
-    stream->SetStreamFile(std::make_unique<AP4_File>(
-        *stream->GetAdByteStream(), AP4_DefaultAtomFactory::Instance_, true, movie));
-    movie = stream->GetStreamFile()->GetMovie();
-
-    if (movie == NULL)
-    {
-      LOG::Log(LOGERROR, "No MOOV in stream!");
-      m_session->EnableStream(stream, false);
-      return false;
-    }
-
-    AP4_Track* track = movie->GetTrack(TIDC[stream->m_adStream.get_type()]);
-    if (!track)
-    {
-      if (stream->m_adStream.get_type() == adaptive::AdaptiveTree::SUBTITLE)
-        track = movie->GetTrack(AP4_Track::TYPE_TEXT);
-      if (!track)
-      {
-        LOG::Log(LOGERROR, "No suitable track found in stream");
-        m_session->EnableStream(stream, false);
-        return false;
-      }
-    }
-
-    auto sampleDecrypter =
-        m_session->GetSingleSampleDecryptor(stream->m_adStream.getRepresentation()->pssh_set_);
-    auto caps = m_session->GetDecrypterCaps(stream->m_adStream.getRepresentation()->pssh_set_);
-
-    stream->SetReader(std::make_unique<CFragmentedSampleReader>(
-        stream->GetAdByteStream(), movie, track, streamid, sampleDecrypter, caps));
-  }
-  else
+  if (!reader)
   {
     m_session->EnableStream(stream, false);
     return false;
   }
 
+  uint16_t psshSetPos = stream->m_adStream.getRepresentation()->m_psshSetPos;
+  reader->SetDecrypter(m_session->GetSingleSampleDecryptor(psshSetPos),
+                       m_session->GetDecrypterCaps(psshSetPos));
+
+  stream->SetReader(std::move(reader));
+
+  if (reprContainerType == ContainerType::TS)
+  {
+    // With TS streams the elapsed time would be calculated incorrectly as during the tree refresh,
+    // nextSegment would be deleted by the FreeSegments/newsegments swap. Do this now before the tree refresh.
+    // Also, when reopening a stream (switching reps) the elapsed time would be incorrectly set until the
+    // second segment plays, now force a correct calculation at the start of the stream.
+    m_session->OnSegmentChanged(&stream->m_adStream);
+  }
+
   if (stream->m_info.GetStreamType() == INPUTSTREAM_TYPE_VIDEO)
   {
-    for (uint16_t i(0); i < 16; ++i)
+    for (auto& [streamType, id] : m_IncludedStreams)
     {
-      if (m_IncludedStreams[i])
+      stream->GetReader()->AddStreamType(streamType, id);
+
+      unsigned int sid = id - m_session->GetPeriodId() * 1000;
+
+      CStream* incStream = m_session->GetStream(sid);
+      if (!incStream)
       {
-        stream->GetReader()->AddStreamType(static_cast<INPUTSTREAM_TYPE>(i), m_IncludedStreams[i]);
-        unsigned int sid = m_IncludedStreams[i] - m_session->GetPeriodId() * 1000;
-        CStream* incStream = m_session->GetStream(sid);
-        if (!incStream) {
-          LOG::LogF(LOGERROR, "Cannot get the stream from sid %u", sid);
-        }
-        stream->GetReader()->GetInformation(
-            m_session->GetStream(m_IncludedStreams[i] - m_session->GetPeriodId() * 1000)->m_info);
+        LOG::LogF(LOGERROR, "Cannot get the stream from sid %u", sid);
+      }
+      else
+      {
+        stream->GetReader()->GetInformation(incStream->m_info);
       }
     }
   }
   m_session->EnableStream(stream, true);
-  return stream->GetReader()->GetInformation(stream->m_info) || needRefetch;
+
+  bool isInfoChanged = stream->GetReader()->GetInformation(stream->m_info);
+
+  uint16_t cdmSessionIndex = stream->m_adStream.getRepresentation()->m_psshSetPos;
+
+  if (stream->m_isEncrypted && m_session->IsCDMSessionSecurePath(cdmSessionIndex))
+  {
+    LOG::Log(LOGDEBUG, "OpenStream(%d): Create secure crypto session", streamid);
+
+    // StreamCryptoSession enable the use of ISA VideoCodecAdaptive decoder
+    kodi::addon::StreamCryptoSession cryptoSession;
+    cryptoSession.SetKeySystem(m_session->GetCryptoKeySystem());
+
+    const char* sessionId(m_session->GetCDMSession(cdmSessionIndex));
+    cryptoSession.SetSessionId(sessionId);
+
+    if (m_session->GetDecrypterCaps(cdmSessionIndex).flags &
+        DRM::DecrypterCapabilites::SSD_SUPPORTS_DECODING)
+      stream->m_info.SetFeatures(INPUTSTREAM_FEATURE_DECODE);
+    else
+      stream->m_info.SetFeatures(INPUTSTREAM_FEATURE_NONE);
+
+    if (m_session->GetDecrypterCaps(cdmSessionIndex).flags &
+        DRM::DecrypterCapabilites::SSD_SECURE_DECODER)
+      cryptoSession.SetFlags(STREAM_CRYPTO_FLAG_SECURE_DECODER);
+    else
+      cryptoSession.SetFlags(STREAM_CRYPTO_FLAG_NONE);
+
+    stream->m_info.SetCryptoSession(cryptoSession);
+    isInfoChanged = true;
+  }
+
+  return isInfoChanged;
 }
 
 
@@ -442,15 +339,7 @@ DEMUX_PACKET* CInputStreamAdaptive::DemuxRead(void)
   if (!m_session)
     return NULL;
 
-  if (m_checkChapterSeek)
-  {
-    m_checkChapterSeek = false;
-    if (m_session->GetChapterSeekTime() > 0)
-    {
-      m_session->SeekTime(m_session->GetChapterSeekTime());
-      m_session->ResetChapterSeekTime();
-    }
-  }
+  m_session->OnDemuxRead();
 
   if (~m_failedSeekTime)
   {
@@ -467,10 +356,11 @@ DEMUX_PACKET* CInputStreamAdaptive::DemuxRead(void)
 
     if (m_session->CheckChange())
     {
-      m_lastPts = adaptive::NO_PTS_VALUE;
+      // Adaptive stream has switched stream (representation) quality
+      m_lastPts = PLAYLIST::NO_PTS_VALUE;
       p = AllocateDemuxPacket(0);
       p->iStreamId = DEMUX_SPECIALID_STREAMCHANGE;
-      LOG::Log(LOGDEBUG, "DEMUX_SPECIALID_STREAMCHANGE");
+      LOG::Log(LOGDEBUG, "DEMUX_SPECIALID_STREAMCHANGE (stream quality changed)");
       return p;
     }
 
@@ -493,7 +383,7 @@ DEMUX_PACKET* CInputStreamAdaptive::DemuxRead(void)
         pData += 16;
         std::memcpy(p->cryptoInfo->kid, pData, 16);
         pData += 16;
-        iSize -= (pData - sr->GetSampleData());
+        iSize -= static_cast<AP4_Size>(pData - sr->GetSampleData());
         CryptoInfo cryptoInfo = sr->GetReaderCryptoInfo();
         p->cryptoInfo->numSubSamples = numSubSamples;
         p->cryptoInfo->cryptBlocks = cryptoInfo.m_cryptBlocks;
@@ -514,9 +404,6 @@ DEMUX_PACKET* CInputStreamAdaptive::DemuxRead(void)
         p->iGroupId = 0;
         p->iSize = iSize;
         std::memcpy(p->pData, pData, iSize);
-        //! @todo: on Kodi 21, sending of subtitles packet side data is no longer 
-        //! needed and can be removed
-        sr->SetDemuxPacketSideData(p, m_session);
       }
 
       //LOG::Log(LOGDEBUG, "DTS: %0.4f, PTS:%0.4f, ID: %u SZ: %d", p->dts, p->pts, p->iStreamId, p->iSize);
@@ -532,17 +419,20 @@ DEMUX_PACKET* CInputStreamAdaptive::DemuxRead(void)
     return p;
   }
 
+  // Ends here when GetNextSample fails due to sample reader in EOS state or stream disabled
+  // that could means, in case of multi-periods streams, that segments are ended
+  // in the current period and its needed to switch to the next period
   if (m_session->SeekChapter(m_session->GetChapter() + 1))
   {
-    m_checkChapterSeek = true;
-    m_lastPts = adaptive::NO_PTS_VALUE;
+    // Switched to new period / chapter
+    m_lastPts = PLAYLIST::NO_PTS_VALUE;
     for (unsigned int i(1);
          i <= INPUTSTREAM_MAX_STREAM_COUNT && i <= m_session->GetStreamCount(); ++i)
       EnableStream(i + m_session->GetPeriodId() * 1000, false);
     m_session->InitializePeriod();
     DEMUX_PACKET* p = AllocateDemuxPacket(0);
     p->iStreamId = DEMUX_SPECIALID_STREAMCHANGE;
-    LOG::Log(LOGDEBUG, "DEMUX_SPECIALID_STREAMCHANGE");
+    LOG::Log(LOGDEBUG, "DEMUX_SPECIALID_STREAMCHANGE (chapter changed)");
     return p;
   }
   return NULL;
@@ -619,7 +509,17 @@ int CInputStreamAdaptive::GetChapterCount()
 
 const char* CInputStreamAdaptive::GetChapterName(int ch)
 {
-  return m_session ? m_session->GetChapterName(ch) : 0;
+  if (!m_session)
+    return nullptr;
+
+  //! @todo: m_chapterName is a workaround fix for compiler
+  //! "warning: returning address of local temporary object"
+  //! we have to store the chapter name locally because the pointer returned is used after
+  //! that Kodi make the GetChapterName callback, so it go out of scope. A way to fix this
+  //! is pass the char pointer by using "strdup", but is needed that when kodi make
+  //! GetChapterName callback also "free" the value after his use.
+  m_chapterName = m_session->GetChapterName(ch);
+  return m_chapterName.c_str();
 }
 
 int64_t CInputStreamAdaptive::GetChapterPos(int ch)
@@ -656,7 +556,7 @@ bool CVideoCodecAdaptive::Open(const kodi::addon::VideoCodecInitdata& initData)
 {
   if (!m_session || !m_session->GetDecrypter())
     return false;
-
+ 
   if ((initData.GetCodecType() == VIDEOCODEC_H264 || initData.GetCodecType() == VIDEOCODEC_AV1) &&
       !initData.GetExtraDataSize() && !(m_state & STATE_WAIT_EXTRADATA))
   {
@@ -692,7 +592,7 @@ bool CVideoCodecAdaptive::Open(const kodi::addon::VideoCodecInitdata& initData)
   Adaptive_CencSingleSampleDecrypter* ssd(m_session->GetSingleSampleDecrypter(sessionId));
 
   return m_session->GetDecrypter()->OpenVideoDecoder(
-      ssd, reinterpret_cast<const SSD::SSD_VIDEOINITDATA*>(initData.GetCStructure()));
+      ssd, initData.GetCStructure());
 }
 
 bool CVideoCodecAdaptive::Reconfigure(const kodi::addon::VideoCodecInitdata& initData)
@@ -705,31 +605,8 @@ bool CVideoCodecAdaptive::AddData(const DEMUX_PACKET& packet)
   if (!m_session || !m_session->GetDecrypter())
     return false;
 
-  SSD::SSD_SAMPLE sample{};
-  sample.data = packet.pData;
-  sample.dataSize = packet.iSize;
-  sample.pts = static_cast<int64_t>(packet.pts);
-  if (packet.cryptoInfo) // Is an encrypted demux packet
-  {
-    sample.cryptoInfo.numSubSamples = packet.cryptoInfo->numSubSamples;
-    sample.cryptoInfo.mode = packet.cryptoInfo->mode;
-    sample.cryptoInfo.cryptBlocks = packet.cryptoInfo->cryptBlocks;
-    sample.cryptoInfo.skipBlocks = packet.cryptoInfo->skipBlocks;
-    sample.cryptoInfo.clearBytes = packet.cryptoInfo->clearBytes;
-    sample.cryptoInfo.cipherBytes = packet.cryptoInfo->cipherBytes;
-    sample.cryptoInfo.iv = packet.cryptoInfo->iv;
-    sample.cryptoInfo.ivSize = 16;
-    sample.cryptoInfo.kid = packet.cryptoInfo->kid;
-    sample.cryptoInfo.kidSize = 16;
-    sample.cryptoInfo.flags = packet.cryptoInfo->flags;
-  }
-  else
-  {
-    sample.cryptoInfo.mode = static_cast<uint16_t>(CryptoMode::NONE);
-  }
-
   return m_session->GetDecrypter()->DecryptAndDecodeVideo(
-             dynamic_cast<kodi::addon::CInstanceVideoCodec*>(this), &sample) != SSD::VC_ERROR;
+             dynamic_cast<kodi::addon::CInstanceVideoCodec*>(this), &packet) != VC_ERROR;
 }
 
 VIDEOCODEC_RETVAL CVideoCodecAdaptive::GetPicture(VIDEOCODEC_PICTURE& picture)
@@ -742,8 +619,7 @@ VIDEOCODEC_RETVAL CVideoCodecAdaptive::GetPicture(VIDEOCODEC_PICTURE& picture)
                                      VIDEOCODEC_RETVAL::VC_EOF};
 
   return vrvm[m_session->GetDecrypter()->VideoFrameDataToPicture(
-      dynamic_cast<kodi::addon::CInstanceVideoCodec*>(this),
-      reinterpret_cast<SSD::SSD_PICTURE*>(&picture))];
+      dynamic_cast<kodi::addon::CInstanceVideoCodec*>(this), &picture)];
 }
 
 void CVideoCodecAdaptive::Reset()
